@@ -1,5 +1,5 @@
 /* $XFree86$ */
-/* $XdotOrg$ */
+/* $XdotOrg: driver/xf86-video-sis/src/sis300_accel.c,v 1.23 2006-03-09 06:06:25 anholt Exp $ */
 /*
  * 2D Acceleration for SiS 530, 620, 300, 540, 630, 730.
  *
@@ -61,7 +61,7 @@
 
 #ifdef SIS_USE_EXA
 extern void SiSScratchSave(ScreenPtr pScreen, ExaOffscreenArea *area);
-extern Bool SiSUploadToScreen(PixmapPtr pDst, char *src, int src_pitch);
+extern Bool SiSUploadToScreen(PixmapPtr pDst, int x, int y, int w, int h, char *src, int src_pitch);
 extern Bool SiSUploadToScratch(PixmapPtr pSrc, PixmapPtr pDst);
 extern Bool SiSDownloadFromScreen(PixmapPtr pSrc, int x, int y, int w, int h,
 				char *dst, int dst_pitch);
@@ -921,13 +921,16 @@ SiSPrepareSolid(PixmapPtr pPixmap, int alu, Pixel planemask, Pixel fg)
 	   }
 	}
 
-	dstbase = (CARD32)((unsigned long)pPixmap->devPrivate.ptr -
-				(unsigned long)pSiS->FbBase + HEADOFFSET);
-	/* TODO: If dstbase is not aligned, need to align it and fix x coordinates */
-	pSiS->fillXoffs = 0;
+	/* Check that the pitch matches the hardware's requirements. Should
+	 * never be a problem due to pixmapPitchAlign and fbScreenInit.
+	 */
+	if(exaGetPixmapPitch(pPixmap) & 3)
+	   return FALSE;
+
+	dstbase = (CARD32)exaGetPixmapOffset(pPixmap) + HEADOFFSET;
 
 	SiSSetupPATFG(fg)
-	SiSSetupDSTRect((pPixmap->devKind + 3) & ~3, -1)
+	SiSSetupDSTRect(exaGetPixmapPitch(pPixmap), -1)
 	SiSSetupDSTColorDepth(dstcol[pPixmap->drawable.bitsPerPixel >> 4]);
 	SiSSetupROP(SiSGetPatternROP(alu))
 	SiSSetupDSTBase(dstbase)
@@ -942,9 +945,6 @@ SiSSolid(PixmapPtr pPixmap, int x1, int y1, int x2, int y2)
 	ScrnInfoPtr pScrn = xf86Screens[pPixmap->drawable.pScreen->myNum];
 	SISPtr pSiS = SISPTR(pScrn);
 
-	x1 += pSiS->fillXoffs;
-	x2 += pSiS->fillXoffs;
-
 	SiSSetupDSTXY(x1, y1)
 	SiSSetupRect(x2-x1, y2-y1)
 
@@ -983,9 +983,17 @@ SiSPrepareCopy(PixmapPtr pSrcPixmap, PixmapPtr pDstPixmap, int xdir, int ydir,
 		  (pDstPixmap->drawable.bitsPerPixel != 32))
 	   return FALSE;
 
+	/* Check that the pitch matches the hardware's requirements. Should
+	 * never be a problem due to pixmapPitchAlign and fbScreenInit.
+	 */
+	if(exaGetPixmapPitch(pSrcPixmap) & 3)
+	   return FALSE;
+	if(exaGetPixmapPitch(pDstPixmap) & 3)
+	   return FALSE;
+
 	SiSSetupDSTColorDepth(dstcol[pDstPixmap->drawable.bitsPerPixel >> 4]);
-	SiSSetupSRCPitch((pSrcPixmap->devKind + 3) & ~3)
-	SiSSetupDSTRect((pDstPixmap->devKind + 3) & ~3, -1)
+	SiSSetupSRCPitch(exaGetPixmapPitch(pSrcPixmap))
+	SiSSetupDSTRect(exaGetPixmapPitch(pDstPixmap), -1)
 
 	SiSSetupROP(SiSGetCopyROP(alu))
 
@@ -996,15 +1004,9 @@ SiSPrepareCopy(PixmapPtr pSrcPixmap, PixmapPtr pDstPixmap, int xdir, int ydir,
 	   SiSSetupCMDFlag(Y_INC)
 	}
 
-	srcbase = (CARD32)((unsigned long)pSrcPixmap->devPrivate.ptr -
-				(unsigned long)pSiS->FbBase + HEADOFFSET);
-	/* TODO: If srcbase is not aligned, need to align it and fix x coordinates */
-	pSiS->copySXoffs = 0;
+	srcbase = (CARD32)exaGetPixmapOffset(pSrcPixmap) + HEADOFFSET;
 
-	dstbase = (CARD32)((unsigned long)pDstPixmap->devPrivate.ptr -
-				(unsigned long)pSiS->FbBase + HEADOFFSET);
-	/* TODO: If dstbase is not aligned, need to align it and fix x coordinates */
-	pSiS->copyDXoffs = 0;
+	dstbase = (CARD32)exaGetPixmapOffset(pDstPixmap) + HEADOFFSET;
 
 	SiSSetupSRCBase(srcbase);
 	SiSSetupDSTBase(dstbase);
@@ -1018,9 +1020,6 @@ SiSCopy(PixmapPtr pDstPixmap, int srcX, int srcY, int dstX, int dstY, int width,
 	ScrnInfoPtr pScrn = xf86Screens[pDstPixmap->drawable.pScreen->myNum];
 	SISPtr pSiS = SISPTR(pScrn);
 
-	srcX += pSiS->copySXoffs;
-	dstX += pSiS->copyDXoffs;
-
 	if(!(pSiS->CommandReg & X_INC))  {
 	   srcX += width-1;
 	   dstX += width-1;
@@ -1102,7 +1101,7 @@ SiS300AccelInit(ScreenPtr pScreen)
 #endif
 #ifdef SIS_USE_EXA
 	   if(pSiS->useEXA) {
-	      if(!(pSiS->EXADriverPtr = xnfcalloc(sizeof(ExaDriverRec), 1))) {
+	      if(!(pSiS->EXADriverPtr = exaDriverAlloc())) {
 		 pSiS->NoAccel = TRUE;
 		 pSiS->NoXvideo = TRUE; /* No fbmem manager -> no xv */
 	      }
@@ -1234,48 +1233,51 @@ SiS300AccelInit(ScreenPtr pScreen)
 #ifdef SIS_USE_EXA	/* ----------------------- EXA ----------------------- */
 	   if(pSiS->useEXA) {
 
+	      pSiS->EXADriverPtr->exa_major = 2;
+	      pSiS->EXADriverPtr->exa_minor = 0;
+
 	      if(pSiS->scrnOffset < 8192) {
 
 		 /* data */
-		 pSiS->EXADriverPtr->card.memoryBase = pSiS->FbBase;
-		 pSiS->EXADriverPtr->card.memorySize = pSiS->maxxfbmem;
-		 pSiS->EXADriverPtr->card.offScreenBase = pScrn->virtualX * pScrn->virtualY
+		 pSiS->EXADriverPtr->memoryBase = pSiS->FbBase;
+		 pSiS->EXADriverPtr->memorySize = pSiS->maxxfbmem;
+		 pSiS->EXADriverPtr->offScreenBase = pScrn->virtualX * pScrn->virtualY
 						* ((pScrn->bitsPerPixel + 7) / 8);
-		 if(pSiS->EXADriverPtr->card.memorySize > pSiS->EXADriverPtr->card.offScreenBase) {
-		    pSiS->EXADriverPtr->card.flags = EXA_OFFSCREEN_PIXMAPS;
+		 if(pSiS->EXADriverPtr->memorySize > pSiS->EXADriverPtr->offScreenBase) {
+		    pSiS->EXADriverPtr->flags = EXA_OFFSCREEN_PIXMAPS;
 		 } else {
 		    pSiS->NoXvideo = TRUE;
 		    xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
 			"Not enough video RAM for offscreen memory manager. Xv disabled\n");
 		 }
-		 pSiS->EXADriverPtr->card.offscreenByteAlign = 16;	/* src/dst: double quad word boundary */
-		 pSiS->EXADriverPtr->card.offscreenPitch = 4;		/* pitch:   double word boundary      */
+		 pSiS->EXADriverPtr->pixmapOffsetAlign = 16;	/* src/dst: double quad word boundary */
+		 pSiS->EXADriverPtr->pixmapPitchAlign = 4;		/* pitch:   double word boundary      */
 		 if(pSiS->VGAEngine == SIS_300_VGA) {
-		    pSiS->EXADriverPtr->card.maxX = 4095;
-		    pSiS->EXADriverPtr->card.maxY = 4095;
+		    pSiS->EXADriverPtr->maxX = 4095;
+		    pSiS->EXADriverPtr->maxY = 4095;
 		 } else {
-		    pSiS->EXADriverPtr->card.maxX = 2047;
-		    pSiS->EXADriverPtr->card.maxY = 2047;
+		    pSiS->EXADriverPtr->maxX = 2047;
+		    pSiS->EXADriverPtr->maxY = 2047;
 		 }
 
 		 /* Sync */
-		 pSiS->EXADriverPtr->accel.WaitMarker = SiSEXASync;
+		 pSiS->EXADriverPtr->WaitMarker = SiSEXASync;
 
 		 /* Solid fill */
-		 pSiS->EXADriverPtr->accel.PrepareSolid = SiSPrepareSolid;
-		 pSiS->EXADriverPtr->accel.Solid = SiSSolid;
-		 pSiS->EXADriverPtr->accel.DoneSolid = SiSDoneSolid;
+		 pSiS->EXADriverPtr->PrepareSolid = SiSPrepareSolid;
+		 pSiS->EXADriverPtr->Solid = SiSSolid;
+		 pSiS->EXADriverPtr->DoneSolid = SiSDoneSolid;
 
 		 /* Copy */
-		 pSiS->EXADriverPtr->accel.PrepareCopy = SiSPrepareCopy;
-		 pSiS->EXADriverPtr->accel.Copy = SiSCopy;
-		 pSiS->EXADriverPtr->accel.DoneCopy = SiSDoneCopy;
+		 pSiS->EXADriverPtr->PrepareCopy = SiSPrepareCopy;
+		 pSiS->EXADriverPtr->Copy = SiSCopy;
+		 pSiS->EXADriverPtr->DoneCopy = SiSDoneCopy;
 
 		 /* Composite not supported */
 
 		 /* Upload, download to/from Screen */
-		 pSiS->EXADriverPtr->accel.UploadToScreen = SiSUploadToScreen;
-		 pSiS->EXADriverPtr->accel.DownloadFromScreen = SiSDownloadFromScreen;
+		 pSiS->EXADriverPtr->UploadToScreen = SiSUploadToScreen;
+		 pSiS->EXADriverPtr->DownloadFromScreen = SiSDownloadFromScreen;
 
 	      } else {
 
@@ -1370,7 +1372,7 @@ SiS300AccelInit(ScreenPtr pScreen)
 						SiSScratchSave, pSiS);
 	      if(pSiS->exa_scratch) {
 		 pSiS->exa_scratch_next = pSiS->exa_scratch->offset;
-		 pSiS->EXADriverPtr->accel.UploadToScratch = SiSUploadToScratch;
+		 pSiS->EXADriverPtr->UploadToScratch = SiSUploadToScratch;
 	      }
 
 	   } else {
