@@ -1,5 +1,5 @@
 /* $XFree86$ */
-/* $XdotOrg$ */
+/* $XdotOrg: driver/xf86-video-sis/src/sis6326_video.c,v 1.19 2005/10/16 18:12:32 twini Exp $ */
 /*
  * Xv driver for SiS 5597/5598, 6326 and 530/620.
  *
@@ -50,6 +50,7 @@
 #define SIS_NEED_outSISREG
 #define SIS_NEED_inSISIDXREG
 #define SIS_NEED_outSISIDXREG
+#define SIS_NEED_setSISIDXREGmask
 #include "sis_regs.h"
 
 #define OFF_DELAY   	200  /* milliseconds */
@@ -71,7 +72,8 @@ static void 	SIS6326QueryBestSize(ScrnInfoPtr, Bool, short, short, short,
 			short, unsigned int *,unsigned int *, pointer);
 static int 	SIS6326PutImage( ScrnInfoPtr,
 			short, short, short, short, short, short, short, short,
-			int, unsigned char*, short, short, Bool, RegionPtr, pointer);
+			int, unsigned char*, short, short, Bool, RegionPtr, pointer,
+			DrawablePtr);
 static int 	SIS6326QueryImageAttributes(ScrnInfoPtr,
 			int, unsigned short *, unsigned short *, int *, int *);
 static void 	SIS6326VideoTimerCallback(ScrnInfoPtr pScrn, Time now);
@@ -1230,7 +1232,8 @@ SIS6326PutImage(
   int id, unsigned char* buf,
   short width, short height,
   Bool sync,
-  RegionPtr clipBoxes, pointer data
+  RegionPtr clipBoxes, pointer data,
+  DrawablePtr pDraw
 ){
    SISPtr pSiS = SISPTR(pScrn);
    SISPortPrivPtr pPriv = (SISPortPrivPtr)data;
